@@ -15,6 +15,7 @@ import requests
 from .compat import OK
 from .description import CLIENT_NAME, SNOWFLAKE_CONNECTOR_VERSION
 from .secret_detector import SecretDetector
+from .test_util import ENABLE_TELEMETRY_LOG, rt_plain_logger
 
 logger = logging.getLogger(__name__)
 
@@ -391,7 +392,11 @@ class TelemetryService(object):
             if not self.is_deployment_enabled():
                 logger.debug("Skip the disabled deployment: %s", self.deployment.name)
                 return
-            logger.debug("Sending OOB telemetry data")
+            logger.debug("Sending OOB telemetry data. Payload: {}".format(payload))
+            if ENABLE_TELEMETRY_LOG:
+                # This logger guarantees the payload won't be masked. Testing purpose.
+                rt_plain_logger.debug("OOB telemetry data being sent is {}".format(payload))
+
             with requests.Session() as session:
                 headers = {
                     'Content-type': 'application/json',
